// copy of https://github.com/fuze/meetings/blob/fcc0317d865dd14bb3625f04ab3669fd77ad4e39/Clients/html5/webroot/js/fuze/util/Log.js
var LogLevel = {
  DEBUG: {value: 100, color: '#999999', prefix: '[DEBUG]'},
  INFO: {value: 200, color: '#1A870E', prefix: '[INFO]'},
  ERROR: {value: 300, color: '#E81010', prefix: '[ERROR]'},
  WARN: {value: 301, color: '#CE7216', prefix: '[WARN]'},
  TRACE: {value: 400, color: '#3472AB', prefix: '[TRACE]'}
};

var LogTarget = {
  BROWSER: {
    append: function (msg, color) {
      if (this.supportsConsoleStyles()) {
        console.log('%c' + msg, 'color:' + color);
      } else {
        console.log(msg);
      }
    },
    supportsConsoleStyles: function () {
      // is webkit? http://stackoverflow.com/a/16459606/376773
      return ('WebkitAppearance' in document.documentElement.style) ||
        // is firebug? http://stackoverflow.com/a/398120/376773
        (window.console && (console.firebug || (console.exception && console.table))) ||
        // is firefox >= v31?
        // https://developer.mozilla.org/en-US/docs/Tools/Web_Console#Styling_messages
        (navigator.userAgent.toLowerCase().match(/firefox\/(\d+)/) && parseInt(RegExp.$1, 10) >= 31);
    }
  }
};

var Log = function () {
};

Log.enabled = true;
Log.appVersion = '';
Log.level = LogLevel.DEBUG;
Log.target = LogTarget.BROWSER;
Log.prefix = '[NGBROWSE]';
Log.messageBuffer = [];

Log.MAX_BUFFER_SIZE = 500;

Log._log = function (level, args, isNeedDisplayingInConsole) {
  if (!Log.enabled) {
    return;
  }
  var msg = null;

  if (args.length > 1) {
    msg = Log._printf.apply(this, args);//handle printf like format
  } else {
    msg = (args.length === 1) ? args[0] : '';
  }

  if (JSUtil.isNullOrUndefined(msg)) {
    msg = (JSUtil.isNull(msg)) ? 'null' : 'undefined';
  }

  var msgText = new StringBuilder();

  if (JSUtil.isObject(msg) && JSUtil.isUndefined(msg.jquery)) {
    msgText.append('\n');
    msgText.append(JSON.stringify(msg, null, 4));
  } else {
    msgText.append(msg);
  }
  var msgFmt = Log.prefix + ' ' + level.prefix + ' ' + DateUtil.formatTimeLong(
    new Date()) + ' v' + Log.appVersion + ' ' + msgText.toString();

  //Log everything into the messageBuffer so we can get the full log
  //when they are send through the feedback form or the silent log upload
  if (Log.messageBuffer.length > Log.MAX_BUFFER_SIZE) {
    Log.messageBuffer.shift();
  }
  Log.messageBuffer.push(msgFmt);

  if (Log.target && isNeedDisplayingInConsole) {
    if (Log.level.value <= level.value) {
      Log.target.append(msgFmt, level.color);
    }
  }
};

Log._printf = function () {

  if (arguments.length === 0) {
    return '';
  }

  var matches = arguments[0].match(/%[@bcdefosuxX]/g);

  if (!matches) {//nothing matched return the original string
    return arguments[0];
  }

  var chunks = arguments[0].split(/%[@bcdefosuxX]/);
  var result = chunks[0];

  for (var i = 0; i < matches.length; i++) {
    var match = matches[i];
    var chunk = chunks[i + 1];
    var arg = arguments[i + 1];
    switch (match) {
      case '%b':
        arg = parseInt(arg).toString(2);
        break;
      case '%c':
        arg = '';
        break; //ignore %c (color notation)
      case '%d':
        arg = parseInt(arg, 10);
        break;
      case '%e':
        arg = parseInt(arg).toExponential();
        break;
      case '%f':
        arg = parseFloat(arg);
        break;
      case '%o':
        arg = parseInt(arg).toString(8);
        break;
      case '%s':
        arg = String(arg);
        break;
      case '%u':
        arg = parseInt(arg) >>> 0;
        break;
      case '%x':
        arg = '0x' + parseInt(arg).toString(16);
        break;
      case '%X':
        arg = '0x' + parseInt(arg).toString(16).toUpperCase();
        break;
      case '%@':
        arg = '\n' + JSON.stringify(arg, null, 4) + '\n';
        break;
    }
    result = result + arg + chunk;
  }
  return result;
};

Log.dump = function (newLine) {
  newLine = newLine || '\n';
  var sb = new StringBuilder();
  for (var i = 0; i < Log.messageBuffer.length; ++i) {
    sb.append(Log.messageBuffer[i]).append(newLine);
  }
  return sb.toString();
};

Log.info = function () {
  Log._log(LogLevel.INFO, arguments);
};

Log.warn = function () {
  Log._log(LogLevel.WARN, arguments);
};

Log.error = function () {
  Log._log(LogLevel.ERROR, arguments);
};

Log.debug = function () {
  Log._log(LogLevel.DEBUG, arguments);
};

Log.trace = function () {
  Log._log(LogLevel.TRACE, arguments);
};
